/**
 * @file        ADS0-52383
 * @description Verify period tasks with filter by production planning checkbox
 * @author      Pethaperumal Natarajan (Pethaperumal.NATARAJAN.intern@3ds.com)
 * @copyright   Dassault Systèmes
 */

import { AppMP, Demo, Scenario } from '../../libmp/appmp';
import { qCustomMatcher } from '../../e2elib/lib/src/main/qmatchers.const';
import { CellLocator } from '../../libappbase/matrixeditorbase';
import { startOfPlanningYear } from '../../libmp/data/data.period';
import { DataMatrixProductPlanningRowName } from '../../libmp/data/data.supplyplanning';
import { StepMatrix } from '../../libappsop/matrixeditorsop';
import { MatrixEditorProductPlanning } from '../../libmp/forms/form.productplanning';
import { StepList } from '../../libappsop/listsop';
import { StepNavigationPanel } from '../../libmp/forms/navigationpanel/form.navigationpanel';
import { DataMetalBaseProductName } from '../../libmp/data/data.navigation';
import { MatrixEditorProductionPlan } from '../../libmp/forms/form.productionplan';
import { StepDialog } from '../../libappsop/dialogsop';
import { dataMetalsProducts, dataMetalsProductsProvider } from '../../libmp/data/data.product';
import { dataMetalsUnits, dataMetalsUnitsProvider } from '../../libmp/data/data.unit';
import { FormProductionPlanDetails, FormProductionPlanningDetailsFields, ListProductionPlanDetails } from '../../libmp/forms/form.productionplandetails';

describe('ADS0-52383 - Verify period tasks with filter by production planning checkbox', () => {
  const appMP = AppMP.getInstance();
  const formProductionPlanDetails = appMP.viewProductionPlan.formProductionPlanDetails;
  const listProductionPlanDetails = formProductionPlanDetails.listProductionPlanDetails;
  const meProductionPlan = appMP.viewProductionPlan.formProductionPlan.meProductionPlan;
  const formNavigation = appMP.formNavigation;
  const listEntity = formNavigation.listEntity;
  const listProduct = formNavigation.listProduct;
  const asiaUnit = dataMetalsUnits.AsiaUnit.Name;
  const verifyToggleFilterDefaultTrue: FormProductionPlanningDetailsFields = { ToggleFilterByProductionPlanning: true };
  const toggleFilterFalse: FormProductionPlanningDetailsFields = { ToggleFilterByProductionPlanning: false };

  const cellLocation: CellLocator = {
    columnName: `1-Jan-${startOfPlanningYear}`,
    rowName: DataMatrixProductPlanningRowName.CPNCoilInQuentin,
  };

  beforeAll(async () => {
    jasmine.addMatchers(qCustomMatcher);
    await appMP.login();
  });

  afterAll(async () => {
    await appMP.resetActiveView(appMP.viewProductionPlan);
    await appMP.cleanupAndLogout();
  });

  afterEach(async () => {
    await appMP.checkToastMessage();
  });

  it(`Step 1 - ${AppMP.getDemoDataPath(Demo.Metals, Scenario.Base)}`, async () => {
    // Wait for optimizer to finish as we need verify the plan
    await appMP.createDemoDataset(Demo.Metals, Scenario.Base, true);
  });

  it(`Step 2 - Open view ${appMP.viewProductionPlan.viewPath}.`, async () => {
    await appMP.viewProductionPlan.switchTo();
  });

  it(`Step 3 - ${StepDialog.verifyDialogValues(FormProductionPlanDetails.title, verifyToggleFilterDefaultTrue)}`, async () => {
    await formProductionPlanDetails.cbFilterByProductionPlanning.verifyValue('true');
  });

  it(`Step 4 - ${StepMatrix.selectCells(MatrixEditorProductPlanning.title, [cellLocation])}`, async () => {
    await meProductionPlan.selectCells([cellLocation]);
  });

  it(`Step 5 - ${StepList.verifyTotalRow(ListProductionPlanDetails.title, 1)}`, async () => {
    await listProductionPlanDetails.verifyTotalRow(1);
  });

  it(`Step 6 - ${StepDialog.updateDialogValues(FormProductionPlanDetails.title, toggleFilterFalse)}`, async () => {
    await formProductionPlanDetails.cbFilterByProductionPlanning.toggle(false);
  });

  it(`Step 7 - ${StepNavigationPanel.showEntitiesList()}`, async () => {
    await formNavigation.toggleEntityList(true);
  });

  it(`Step 8 - ${StepNavigationPanel.checkUnits([asiaUnit])}`, async () => {
    const row = await listEntity.getRow([dataMetalsUnitsProvider, dataMetalsUnits.AsiaUnit]);
    await listEntity.toggleRowCheckbox(row, true);
  });

  it(`Step 9 - ${StepNavigationPanel.showProductsList()}`, async () => {
    await formNavigation.toggleProductList(true);
  });

  it(`Step 10 - ${StepNavigationPanel.checkProducts([DataMetalBaseProductName.BCCBWide])}`, async () => {
    const [bccbwide, parents] = listProduct.getHierarchy(dataMetalsProductsProvider, dataMetalsProducts.BCCBWide);
    const row = await listProduct.getRow(bccbwide, parents);
    await listProduct.toggleRowCheckbox(row, true);
  });

  it(`Step 11 - ${StepMatrix.verifyTotalRow(MatrixEditorProductionPlan.title, 1)} ${StepMatrix.verifyRowExists(MatrixEditorProductionPlan.title, DataMatrixProductPlanningRowName.BCCBWideInQiYing)}`, async () => {
    await meProductionPlan.verifyTotalRow(1);
    await meProductionPlan.verifyRowExist(DataMatrixProductPlanningRowName.BCCBWideInQiYing);
  });

  it(`Step 12 - ${StepList.verifyTotalRow(ListProductionPlanDetails.title, 18)}`, async () => {
    await listProductionPlanDetails.verifyTotalRow(18);
  });
});
